# Copyright 2015,  A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from a10_openstack_lib.resources import a10_device_instance

from a10_neutronclient import client_extension


class DeviceInstanceExtension(client_extension.ClientExtension):

    resource = a10_device_instance.RESOURCE
    resource_plural = a10_device_instance.RESOURCES

    resource_attribute_map = a10_device_instance.RESOURCE_ATTRIBUTE_MAP

    object_path = '/%s' % resource_plural
    resource_path = '/%s/%%s' % resource_plural
    versions = ['2.0']


class DeviceInstanceList(client_extension.List, DeviceInstanceExtension):
    """List current A10 vThunder instances"""

    shell_command = 'a10-device-instance-list'

    list_columns = ['id', 'name', 'host', 'api_version', 'nova_instance_id', 'description']


class DeviceInstanceShow(client_extension.Show, DeviceInstanceExtension):
    """Show A10 vThunder instance"""

    shell_command = 'a10-device-instance-show'


class DeviceInstanceCreate(client_extension.Create, DeviceInstanceExtension):

    shell_command = 'a10-device-instance-create'
    list_columns = ['name', 'host', 'nova_instance_id']


class DeviceInstanceDelete(client_extension.Delete, DeviceInstanceExtension):
    """Delete A10 vThunder Instance"""

    shell_command = 'a10-device-instance-delete'


class DeviceInstanceUpdate(client_extension.Update, DeviceInstanceExtension):
    """Update A10 vThunder Instance"""

    shell_command = "a10-device-instance-update"
    list_columns = ["name", "host", "nova_instance_id", "api_version"]



# Copyright (C) 2016 A10 Networks Inc. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from a10_openstack_lib.resources import a10_certificate

from a10_neutronclient import client_extension


class CertificateExtension(client_extension.ClientExtension):

    resource = a10_certificate.CERTIFICATE
    resource_plural = a10_certificate.CERTIFICATES

    resource_attribute_map = a10_certificate.RESOURCE_ATTRIBUTE_MAP

    object_path = '/%s' % resource_plural
    resource_path = '/%s/%%s' % resource_plural
    versions = ['2.0']


class CertificateList(client_extension.List, CertificateExtension):
    """List A10 SSL Certificates"""

    shell_command = 'a10-certificate-list'

    list_columns = ['id', 'name', 'description']


class CertificateCreate(client_extension.Create, CertificateExtension):
    """Create A10 SSL Certificate"""

    shell_command = 'a10-certificate-create'

    list_columns = ['id', 'name', 'description']

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--cert-file',
            action=client_extension.ReadFileAction,
            dest='cert_data')

        parser.add_argument(
            '--key-file',
            action=client_extension.ReadFileAction,
            dest='key_data')

        parser.add_argument(
            '--intermediate-file',
            action=client_extension.ReadFileAction,
            dest='intermediate_data')
        self._add_known_arguments(parser, ['name'])


class CertificateDelete(client_extension.Delete, CertificateExtension):
    """Delete A10 SSL Certificate"""

    shell_command = 'a10-certificate-delete'


class CertificateShow(client_extension.Show, CertificateExtension):
    """Show A10 SSL Certificate"""

    shell_command = 'a10-certificate-show'


class CertificateUpdate(client_extension.Update, CertificateExtension):
    """Update A10 SSL Certificate"""

    shell_command = 'a10-certificate-update'

    list_columns = ['id', 'name']


class CertificateBindingExtension(client_extension.ClientExtension):

    resource = a10_certificate.CERTIFICATE_BINDING
    resource_plural = a10_certificate.CERTIFICATE_BINDINGS

    resource_attribute_map = a10_certificate.RESOURCE_ATTRIBUTE_MAP

    object_path = '/%s' % resource_plural
    resource_path = '/%s/%%s' % resource_plural
    versions = ['2.0']


class CertificateBindingList(client_extension.List, CertificateBindingExtension):
    """List A10 A10 SSL Certificate <-> Listener Bindings"""

    shell_command = 'a10-certificatebinding-list'

    list_columns = ['id', 'certificate_id', 'listener_id']


class CertificateBindingCreate(client_extension.Create, CertificateBindingExtension):
    """Create A10 SSL Certificate <-> Listener Binding"""

    shell_command = 'a10-certificatebinding-create'
    list_columns = ['id', 'certificate_id', 'listener_id']

    def add_known_arguments(self, parser):
        self._add_known_arguments(parser, ['certificate_id', 'listener_id'])


class CertificateBindingDelete(client_extension.Delete, CertificateBindingExtension):
    """Delete A10 SSL Certificate <-> Listener Binding"""

    shell_command = 'a10-certificatebinding-delete'


class CertificateBindingShow(client_extension.Show, CertificateBindingExtension):
    """Show A10 SSL Certificate <-> Listener Binding Details"""

    shell_command = 'a10-certificatebinding-show'



# Copyright (C) 2016 A10 Networks Inc. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import argparse

from neutronclient.common import extension
from neutronclient.neutron import v2_0 as neutronV20

_NEUTRON_OPTIONS = ['id', 'tenant_id']

JUST_NONE = object()


class ClientExtension(extension.NeutronClientExtension):

    def _arg_name(self, name, types, prefix="--"):
        if 'type:a10_nullable' in types:
            return self._arg_name(name, types['type:a10_nullable'], prefix)

        if 'type:a10_list' in types:
            return self._arg_name(name, types['type:a10_list'], prefix)

        if 'type:a10_reference' in types:
            if name.endswith('_id'):
                name = name[:-3]

        """--shish-kabob it"""
        return prefix + name.replace('_', '-')

    def _add_known_argument(self, parser, name, types):
        if 'type:a10_nullable' in types:
            parser.add_argument(
                self._arg_name(name, types, '--no-'),
                action='store_const',
                const=JUST_NONE,
                dest=name)

            self._add_known_argument(parser, name, types['type:a10_nullable'])
            return

        if 'type:a10_list' in types:
            parser.add_argument(
                self._arg_name(name, types['type:a10_list']),
                nargs='*',
                action='append',
                dest=name)
            return

        parser.add_argument(self._arg_name(name, types), dest=name)

    def _add_known_arguments(self, parser, required, ignore=[], where=lambda x: True):
        attributes = self.resource_attribute_map[self.resource_plural]
        for name in required:
            parser.add_argument(name)
        for name, attr in attributes.items():
            if name in required or name in _NEUTRON_OPTIONS or name in ignore or not where(attr):
                continue
            types = attr.get('validate', {})

            self._add_known_argument(parser, name, types)

    def alter_body(self, parsed_args, body):
        return body

    def _transform_arg(self, value, types):
        if value == JUST_NONE:
            return None

        if 'type:a10_nullable' in types:
            return self._transform_arg(value, types['type:a10_nullable'])

        if 'type:a10_list' in types:
            # argparse makes lists of lists
            # one sublist per instance of the argument
            return [self._transform_arg(x, types['type:a10_list'])
                    for sublist in value
                    for x in sublist]

        if 'type:a10_reference' in types:
            reference_to = types['type:a10_reference']
            return self.get_resource_id(reference_to, value)

        return value

    def args2body(self, parsed_args):
        attributes = self.resource_attribute_map[self.resource_plural]
        body = {}
        neutronV20.update_dict(parsed_args, body, [a for a in attributes if a != 'id'])

        for k in body:
            types = attributes.get(k, {}).get('validate', {})
            body[k] = self._transform_arg(body[k], types)

        altered = self.alter_body(parsed_args, body)
        return {self.resource: altered}

    def get_resource_id(self, resource, name_or_id):
        client = self.get_client()
        return neutronV20.find_resourceid_by_name_or_id(
            client,
            resource,
            name_or_id)


class List(extension.ClientExtensionList):
    pagination_support = True
    sorting_support = True


class Create(extension.ClientExtensionCreate):

    def _add_known_arguments(self, parser, required, ignore=[]):
        super(Create, self)._add_known_arguments(
            parser,
            required,
            ignore=ignore,
            where=lambda attr: attr.get('allow_post'))

    def add_known_arguments(self, parser):
        return self._add_known_arguments(parser, [])


class Update(extension.ClientExtensionUpdate):

    def _add_known_arguments(self, parser, required, ignore=[]):
        super(Update, self)._add_known_arguments(
            parser,
            required,
            ignore=ignore,
            where=lambda attr: attr.get('allow_put'))

    def add_known_arguments(self, parser):
        return self._add_known_arguments(parser, [])


class Delete(extension.ClientExtensionDelete):
    pass


class Show(extension.ClientExtensionShow):
    pass


class ReadFileAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        with open(values, 'r') as file:
            data = file.read()
            setattr(namespace, self.dest, data)



#!/usr/bin/env python
# flake8: noqa

from setuptools import setup, find_packages
setup(
    name = "a10-neutronclient",
    version = "0.2.0",
    packages = find_packages(),

    author = "A10 Networks",
    author_email = "dougw@a10networks.com",
    description = "A10 Networks Neutron Client Extensions",
    license = "Apache",
    keywords = "a10 adc slb load balancer openstack neutron lbaas cli",
    url = "https://github.com/a10networks/a10-neutronclient",

    long_description = 'A10 neutron client extensions',

    classifiers = [
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet',
    ],
    entry_points = {
        'neutronclient.extension': [
            'a10_scaling_group=a10_neutronclient.a10_scaling_group',
            'a10_device_instance=a10_neutronclient.a10_device_instance',
            'a10_certificate=a10_neutronclient.a10_certificate'
        ],
    },
    include_package_data=True,

    install_requires = ['python-neutronclient>=2.3.2', 'a10-openstack-lib>=0.2.0']
)
